"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OnDisconnect = exports.OnEvent = exports.OnConnection = exports.useSocket = exports.loadEvent = exports.socketGroups = exports.socketConfig = exports.socketIo = exports.io = void 0;
const socket_io_1 = require("socket.io");
const common_1 = require("./common");
exports.socketGroups = [];
var SocketChannel;
(function (SocketChannel) {
    SocketChannel["Connection"] = "connection";
    SocketChannel["Event"] = "event";
})(SocketChannel || (SocketChannel = {}));
function loadEvent(config) {
    exports.socketConfig = config;
}
exports.loadEvent = loadEvent;
function useSocket(server, config) {
    exports.io = new socket_io_1.Server(server, config);
    (0, common_1.display)("Socket: Started"); // log message
}
exports.useSocket = useSocket;
function OnConnection() {
    return function (target, key) {
        exports.socketGroups.push({
            channel: SocketChannel.Connection,
            callback: function () {
                exports.io.on("connection", target[key]);
            }
        });
    };
}
exports.OnConnection = OnConnection;
function OnEvent(path) {
    return function (target, key) {
        exports.socketGroups.push({
            channel: SocketChannel.Event,
            callback: function () {
                exports.io.on("connection", (socket) => socket.on(path, (payload) => target[key](payload, socket)));
            }
        });
    };
}
exports.OnEvent = OnEvent;
function OnDisconnect() {
    return function (target, key) {
        exports.socketGroups.push({
            channel: SocketChannel.Event,
            callback: function () {
                exports.io.on("connection", (socket) => socket.on("disconnect", () => target[key](socket)));
            }
        });
    };
}
exports.OnDisconnect = OnDisconnect;
class SocketServer {
    constructor() {
        let countConnection = 0;
        for (let i = 0; i < exports.socketGroups.length; i++) {
            const socket = exports.socketGroups[i];
            if (countConnection > 1)
                throw new Error("OnConnection can be call only once");
            if (socket.channel === SocketChannel.Connection)
                countConnection++;
            socket.callback(); // intial listener
        }
    }
}
exports.default = SocketServer;
